## Plone Web Mail
## Copyright (C) 2004, OPENPROJECTS, http://openprojects.it
## Maintained by Giorgio Pieretti <giorgio.pieretti at openprojects.it>
## Released under GPL version 2
## See LICENSE.txt or http://www.gnu.org/copyleft/gpl.html


from StringIO import StringIO
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.PloneWebMail.config import TOOL_NAME, PKG_NAME, TOOL_META_TYPE, SKIN_NAME
from Products.PloneWebMail import install_globals
from Products.PloneWebMail import PloneWebMail_box,PloneWebMail_addressbook
from Products.CMFCore.TypesTool import ContentFactoryMetadata
from Products.CMFCore import CMFCorePermissions
import string



CMFCorePermissions.setDefaultRoles("Add PloneWebMail_box", ('Manager', 'Owner'))
types_list = ("PloneWebMail_box","PloneWebMail_addressbook",)




def installTool(self, out):
      if not hasattr(self, TOOL_NAME):
          addTool = self.manage_addProduct[PKG_NAME].manage_addTool
          # Add the tool by its meta_type
          addTool(TOOL_META_TYPE)
	  out.write('%s added.\n' % TOOL_NAME)	  
      else:
          out.write('%s already present.\n' % TOOL_NAME)
      


def unregisterActionProvider(self, out):
    actionTool = getToolByName(self, 'portal_actions', None)
    if actionTool:
        actionTool.deleteActionProvider(TOOL_NAME)
        out.write('Removed %s\n' % TOOL_NAME)


def uninstall(self):
    out=StringIO()
    unregisterActionProvider(self, out)
    return out.getvalue()




def setupWorkflows(self, out):
    wf_tool = getToolByName(self, 'portal_workflow')
    wf_tool.setChainForPortalTypes( ('PloneWebMail_box',), '')
    out.write("PloneWebMail_box Worflow set to Null\n")
    wf_tool.setChainForPortalTypes( ('PloneWebMail_addressbook',), '(Default)')
    out.write("PloneWebMail_box Worflow set to (Default)\n")




def installSubSkin(self, out, skinFolder):
    """ Install a subskin, i.e. a folder/directoryview.
    """
    skinsTool = getToolByName(self, 'portal_skins')

    for skin in skinsTool.getSkinSelections():
        path = skinsTool.getSkinPath(skin)
        path = map( string.strip, string.split( path,',' ) )
        if not skinFolder in path:
            try:
                path.insert( path.index( 'custom')+1, skinFolder )
            except ValueError:
                path.append(skinFolder)
            path = string.join( path, ', ' )
            skinsTool.addSkinSelection( skin, path )
            out.write('Subskin successfully installed into %s.\n' % skin)    
        else:
            out.write('*** Subskin was already installed into %s.\n' % skin) 


def setupTypesandSkins(self, out):
    fti_list = (
    PloneWebMail_box.factory_type_information,
    PloneWebMail_addressbook.factory_type_information,
    )

    typesTool = getToolByName(self, 'portal_types')
    skinsTool = getToolByName(self, 'portal_skins')
    for f in fti_list:
        if f['id'] in typesTool.objectIds():
            out.write('*** Object "%s" already existed in the types tool => deleting\n' % (f['id']))

            typesTool._delObject(f['id'])
    for f in fti_list:
        cfm = apply(ContentFactoryMetadata, (), f)
        typesTool._setObject(f['id'], cfm)
        out.write('Type "%s" registered with the types tool\n' % (f['id']))
    try:
        addDirectoryViews(skinsTool, 'skins', install_globals)
        out.write( "Added directory views to portal_skins.\n" )
    except:
        out.write( '*** Unable to add directory views to portal_skins.\n')
    installSubSkin(self, out, SKIN_NAME)


def addNavigationTransitions(self):
    portal_prop = getToolByName(self, 'portal_properties')
    form = portal_prop.form_properties
    nav = portal_prop.navigation_properties
    for ctype in types_list:
        lowertype = ctype.lower()
        mapping = [
            ('%s_editForm' % (ctype), 'validate_id,validate_%s_edit' % (ctype), ),
            ]
        
        for (name, value) in mapping:
            try:
                form._setProperty(name, value, 'string')
            except:
                form._updateProperty(name, value)

        mapping = [
            ('%s.%s_editForm.success' % (lowertype, ctype ), 'script:%s_edit' % (ctype), ),
            ('%s.%s_editForm.failure' % (lowertype, ctype ), '%s_editForm' % (ctype), ),
            ('%s.%s_edit.success' % (lowertype, ctype ), 'action:view', ),
            ('%s.%s_edit.failure' % (lowertype, ctype ), 'action:edit', ),
            ]

        for (name, value) in mapping:
            try:
                nav._setProperty(name, value, 'string')
            except:
                nav._updateProperty(name, value)


def setupSlots(self, out):
    left_slots = list(getattr(self, 'left_slots'))
    if 'here/mailboxes_slot/macros/mailboxes' not in left_slots:
        left_slots.append('here/mailboxes_slot/macros/mailboxes')
        self._updateProperty('left_slots', left_slots)
        out.write('Added PloneWebMail Slot to Portal.\n')
    else:
        out.write('PloneWebMail Slot already present.\n')



def install(self):
    out=StringIO()
    setupTypesandSkins(self, out)
    addNavigationTransitions(self)
    setupSlots(self, out)
    setupWorkflows(self, out)
    installTool(self, out)
    out.write('Installation completed.\n')
    return out.getvalue()

